// Copyright Vrai Stacey, 2008
// $Id: birtdialog.cpp 1467 2008-11-18 21:57:44Z vrai $

#include "birtdialog.h"
#include "progressdialog.h"
#include "searchingdialog.h"
#include "errorsdialog.h"
#include "helpdialog.h"
#include "imageformats.h"
#include <QMessageBox>
#include <QCloseEvent>
#include <QFileDialog>
#include <QSettings>

BirtDialog::BirtDialog ( )
    : QDialog ( )
    , m_progressDialog ( new ProgressDialog ( this ) )
    , m_searchingDialog ( new SearchingDialog ( this ) )
    , m_errorsDialog ( new ErrorsDialog ( this ) )
    , m_helpDialog ( new HelpDialog ( this ) )
    , m_imageResizer ( new ImageResizer ( this ) )
    , m_fileSearcher ( new FileSearcher ( this ) )
    , m_lastDirSearchRoot ( QDir::current ( ) )
    , m_lastOpenFileRoot ( QDir::current ( ) )
{
    m_ui.setupUi ( this );

    // Set the dialog icon
    setWindowIcon ( QIcon ( ":/images/birt_icon_48x48.png" ) );

    // Wire up the main dialog buttons
    connect ( m_ui.quitButton,              SIGNAL( clicked ( ) ),                                  this,                         SLOT( close ( ) ) );
    connect ( m_ui.helpButton,              SIGNAL( clicked ( ) ),                                  this,                         SLOT( displayHelp ( ) ) );
    connect ( m_ui.addFilesButton,          SIGNAL( clicked ( ) ),                                  this,                         SLOT( addFileToList ( ) ) );
    connect ( m_ui.removeFilesButton,       SIGNAL( clicked ( ) ),                                  this,                         SLOT( removeFilesFromList ( ) ) );
    connect ( m_ui.goButton,                SIGNAL( clicked ( ) ),                                  this,                         SLOT( processFiles ( ) ) );
    connect ( m_ui.addTreeButton,           SIGNAL( clicked ( ) ),                                  this,                         SLOT( searchDirTree ( ) ) );
    connect ( m_ui.removeAllButton,         SIGNAL( clicked ( ) ),                                  &m_fileListModel,             SLOT( removeAll ( ) ) );
    connect ( m_ui.clearButton,             SIGNAL( clicked ( ) ),                                  m_ui.filesListView,           SLOT( clearSelection ( ) ) );
    connect ( m_ui.browseButton,            SIGNAL( clicked ( ) ),                                  m_ui.outputDirectoryLineEdit, SLOT( browseClicked ( ) ) );

    // Wire up the JPEG quality slider to the model and the quality label
    connect ( m_ui.jpegQualitySlider,       SIGNAL( valueUpdated ( int ) ),                         m_ui.jpegQualityLabel,        SLOT( setNum ( int ) ) );
    connect ( m_ui.jpegQualitySlider,       SIGNAL( valueUpdated ( int ) ),                         &m_transformationModel,       SLOT( changedJpegQuality ( int ) ) );

    // Wire up the various file selection buttons to the "files have been added/removed" signal (this triggers the
    // buttons to enable/disable)
    connect ( &m_fileListModel,             SIGNAL( fileCountChanged ( int ) ),                     m_ui.removeAllButton,         SLOT( fileCountChanged ( int ) ) );
    connect ( &m_fileListModel,             SIGNAL( fileCountChanged ( int ) ),                     m_ui.clearButton,             SLOT( fileCountChanged ( int ) ) );
    connect ( &m_fileListModel,             SIGNAL( fileCountChanged ( int ) ),                     m_ui.removeFilesButton,       SLOT( fileCountChanged ( int ) ) );
    connect ( &m_fileListModel,             SIGNAL( fileCountChanged ( int ) ),                     m_ui.goButton,                SLOT( fileCountChanged ( int ) ) );

    // Wire up more file selection butons to the "files have been selected/deselected" signal (same efect as above)
    connect ( m_ui.filesListView,           SIGNAL( changedSelection ( int ) ),                     m_ui.clearButton,             SLOT( changedSelection ( int ) ) );
    connect ( m_ui.filesListView,           SIGNAL( changedSelection ( int ) ),                     m_ui.removeFilesButton,       SLOT( changedSelection ( int ) ) );

    // Wire up the transformation model inputs to the model
    connect ( m_ui.widthLineEdit,           SIGNAL( textChanged ( const QString & ) ),              getBridge ( "width" ),        SLOT( single ( const QString & ) ) );
    connect ( m_ui.heightLineEdit,          SIGNAL( textChanged ( const QString & ) ),              getBridge ( "height" ),       SLOT( single ( const QString & ) ) );
    connect ( getBridge ( "width" ),        SIGNAL( single ( int ) ),                               &m_transformationModel,       SLOT( changedWidth ( int ) ) );
    connect ( getBridge ( "height" ),       SIGNAL( single ( int ) ),                               &m_transformationModel,       SLOT( changedHeight ( int ) ) );
    connect ( m_ui.ratioComboBox,           SIGNAL( changedRatio ( RatioStrategy::Strategy ) ),     &m_transformationModel,       SLOT( changedRatioStrategy ( RatioStrategy::Strategy ) ) );
    connect ( m_ui.scalingComboBox,         SIGNAL( changedScaling ( Qt::TransformationMode ) ),    &m_transformationModel,       SLOT( changedScalingMode ( Qt::TransformationMode ) ) );

    // Wire up the output model inputs to the model
    connect ( m_ui.outputDirectoryLineEdit, SIGNAL( directoryChanged ( const QDir & ) ),            &m_outputModel,               SLOT( changedOutputDirectory ( const QDir & ) ) );
    connect ( m_ui.outputPrefixLineEdit,    SIGNAL( textChanged ( const QString & ) ),              &m_outputModel,               SLOT( changedPrefix ( const QString & ) ) );
    connect ( m_ui.outputSuffixLineEdit,    SIGNAL( textChanged ( const QString & ) ),              &m_outputModel,               SLOT( changedSuffix ( const QString & ) ) );
    connect ( m_ui.formatComboBox,          SIGNAL( changedFormat ( ) ),                            &m_outputModel,               SLOT( changedFormat ( ) ) );
    connect ( m_ui.formatComboBox,          SIGNAL( changedFormat ( const QString & ) ),            &m_outputModel,               SLOT( changedFormat ( const QString & ) ) );

    // Wire up the Image Resizer object to the "images are being resized" dialog box
    connect ( m_imageResizer,               SIGNAL( started ( ) ),                                  m_progressDialog,             SLOT( show ( ) ) );
    connect ( m_imageResizer,               SIGNAL( finished ( ) ),                                 m_progressDialog,             SLOT( hide ( ) ) );
    connect ( m_imageResizer,               SIGNAL( updateCount ( int, int ) ),                     m_progressDialog,             SLOT( countUpdated ( int, int ) ) );
    connect ( m_progressDialog,             SIGNAL( rejected ( ) ),                                 m_imageResizer,               SLOT( abort ( ) ) );

    // Wire up the Directory Search object to the "searching directories" dialog box
    connect ( m_fileSearcher,               SIGNAL( started ( ) ),                                  m_searchingDialog,            SLOT( show ( ) ) );
    connect ( m_fileSearcher,               SIGNAL( finished ( ) ),                                 m_searchingDialog,            SLOT( hide ( ) ) );
    connect ( m_fileSearcher,               SIGNAL( updateCount ( unsigned long, unsigned long ) ), m_searchingDialog,            SLOT( countUpdated ( unsigned long, unsigned long ) ) );
    connect ( m_fileSearcher,               SIGNAL( updateDirectory ( QString ) ),                  m_searchingDialog,            SLOT( directoryUpdated ( QString ) ) );
    connect ( m_searchingDialog,            SIGNAL( rejected ( ) ),                                 m_fileSearcher,               SLOT( abort ( ) ) );

    // Attach a model to the file list
    m_ui.filesListView->setModel ( &m_fileListModel );

    // Initialise the resize option fields
    m_ui.outputDirectoryLineEdit->setText ( QDir::currentPath ( ) );
    m_ui.jpegQualitySlider->setValue ( 95 );
    m_ui.ratioComboBox->setCurrentIndex ( 0 );                          // TODO Add the ability to set by enum
    m_ui.scalingComboBox->setCurrentIndex ( 0 );                        // TODO Ditto
}

void BirtDialog::reject ( )
{
    // Don't pass this on - it should only be trigged by someone pressing escape.
}

void BirtDialog::processFiles ( )
{
    // Check that the models are in a usable state
    QStringList errors;
    if ( m_outputModel.validate ( errors ) && m_transformationModel.validate ( errors ) )
    {
        // Configure the resizer thread and add the files to be processed to it
        m_imageResizer->configure ( m_transformationModel.getDimensions ( ),
                                    m_outputModel.getOutputDirectory ( ),
                                    m_outputModel.getPrefix ( ),
                                    m_outputModel.getSuffix ( ),
                                    m_transformationModel.getScalingMode ( ),
                                    m_transformationModel.getRatioStrategy ( ),
                                    m_transformationModel.getJpegQuality ( ) );
        if ( m_outputModel.changeFormat ( ) )
            m_imageResizer->setFormatOverride ( m_outputModel.getFormat ( ) );

        m_imageResizer->clearFiles ( );
        for ( int rowIndex ( 0 ), rowCount ( m_fileListModel.rowCount ( ) ); rowIndex < rowCount; ++rowIndex )
            m_imageResizer->addFile ( m_fileListModel.getFileInfo ( rowIndex ) );

        // Resize the image
        m_imageResizer->run ( );

        // If there are any errors message - display them in the error dialog
        QHash<QString, QString> errors;
        m_imageResizer->getErrors ( errors );
        if ( ! errors.isEmpty ( ) )
        {
            m_errorsDialog->setErrors ( errors );
            m_errorsDialog->show ( );
        }
    }
    else
    {
        // Create the error message - a static header line followed by each of the errors
        QString message ( tr ( "The are problems with the resize options:\n\n" ) );
        foreach ( const QString & error, errors )
            message += error + "\n";

        // Inform the user and bail
        QMessageBox::critical ( this, tr ( "Cannot resize files" ), message );
    }
}

void BirtDialog::searchDirTree ( )
{
    // Get the root directory for the search
    const QString directory = QFileDialog::getExistingDirectory ( this,
                                                                  tr ( "Select the directory to add" ),
                                                                  m_lastDirSearchRoot.path ( ),
                                                                  QFileDialog::ShowDirsOnly );
    if ( directory.isEmpty ( ) )
        return;

    // Remember the selected directory for next time
    m_lastDirSearchRoot = QDir ( directory );

    // Configure the searcher and start it running (synchronous, but event handling)
    m_fileSearcher->configure ( QDir ( directory ) );
    m_fileSearcher->run ( );

    // Update the file model with searcher's results
    QList<QFileInfo> results;
    m_fileSearcher->getMatchingFiles ( results );
    m_fileListModel.addFiles ( results );
}

void BirtDialog::displayHelp ( )
{
    if ( ! m_helpDialog->isVisible ( ) )
        m_helpDialog->show ( );

    m_helpDialog->setFocus ( );
}

void BirtDialog::addFileToList ( )
{
    // Get a list of image files from the user and add them to the model
    const QStringList files = QFileDialog::getOpenFileNames ( this,
                                                              tr ( "Select image files to be resized" ),
                                                              m_lastOpenFileRoot.path ( ),
                                                              ImageFormats::instance ( ).getFileDialogConfig ( ) );

    // Remember the last directory visited for next time
    if ( ! files.isEmpty ( ) )
        m_lastOpenFileRoot = QFileInfo ( files.last ( ) ).dir ( );

    // Convert the filenames in to file info instances before passing them to the model
    QList<QFileInfo> fileinfo;
    foreach ( const QString & filename, files )
        fileinfo.append ( QFileInfo ( filename ) );

    m_fileListModel.addFiles ( fileinfo );
}

void BirtDialog::removeFilesFromList ( )
{
    // Simply get the selected row indicies from the view and pass them on to the model for deletion
    QVector<int> rows;
    m_ui.filesListView->getSelectedRows ( rows );
    m_fileListModel.removeRows ( rows );
}

void BirtDialog::closeEvent ( QCloseEvent * event )
{
    // In a future version this could be used to prompt before shutdown. For the time being it's used for closing any
    // non-modal child-windows that may be open.
    m_helpDialog->reject ( );

    event->accept ( );
}

StringIntBridge * BirtDialog::getBridge ( const QString & name )
{
    QHash<QString, StringIntBridge *>::iterator bridgeIt ( m_bridges.find ( name ) );
    if ( bridgeIt != m_bridges.end ( ) )
        return bridgeIt.value ( );

    StringIntBridge * bridge ( new StringIntBridge ( this ) );
    return m_bridges [ name ] = bridge;
}

void BirtDialog::store ( ) const
{
    QSettings settings;

    settings.beginGroup ( "transformationModel" );

    settings.setValue ( "ratioStrategy", static_cast<int> ( m_transformationModel.getRatioStrategy ( ) ) );
    settings.setValue ( "scalingMode", static_cast<int> ( m_transformationModel.getScalingMode ( ) ) );
    settings.setValue ( "jpegQuality", m_transformationModel.getJpegQuality ( ) );

    const QSize dimensions ( m_transformationModel.getDimensions ( ) );
    if ( dimensions.width ( ) >= 0 && dimensions.height ( ) >= 0 )
        settings.setValue ( "dimensions", dimensions );

    settings.endGroup ( );
    settings.beginGroup ( "outputModel" );

    settings.setValue ( "outputDirectory", m_outputModel.getOutputDirectory ( ).path ( ) );
    settings.setValue ( "outputPrefix", m_outputModel.getPrefix ( ) );
    settings.setValue ( "outputSuffix", m_outputModel.getSuffix ( ) );
    settings.setValue ( "outputFormatChange", m_outputModel.changeFormat ( ) );
    settings.setValue ( "outputFormatValue", m_outputModel.getFormat ( ) );

    settings.endGroup ( );
}

void BirtDialog::restore ( )
{
    QSettings settings;
    settings.beginGroup ( "transformationModel" );

    if ( settings.contains ( "ratioStrategy" ) ) m_ui.ratioComboBox->setRatioStrategy ( static_cast<RatioStrategy::Strategy> ( settings.value ( "ratioStrategy" ).toInt ( ) ) );
    if ( settings.contains ( "scalingMode" ) )   m_ui.scalingComboBox->setScalingMode ( static_cast<Qt::TransformationMode>  ( settings.value ( "scalingMode" ).toInt ( ) ) );
    if ( settings.contains ( "jpegQuality" ) )   m_ui.jpegQualitySlider->setValue ( settings.value ( "jpegQuality" ).toInt ( ) );

    if ( settings.contains ( "dimensions" ) )
    {
        const QSize dimensions ( settings.value ( "dimensions" ).toSize ( ) );

        m_ui.widthLineEdit->setInt ( dimensions.width ( ) );
        m_ui.heightLineEdit->setInt ( dimensions.height ( ) );
    }

    settings.endGroup ( );
    settings.beginGroup ( "outputModel" );

    if ( settings.contains ( "outputDirectory" ) ) m_ui.outputDirectoryLineEdit->setText ( settings.value ( "outputDirectory" ).toString ( ) );
    if ( settings.contains ( "outputPrefix" ) )    m_ui.outputPrefixLineEdit->setText ( settings.value ( "outputPrefix" ).toString ( ) );
    if ( settings.contains ( "outputSuffix" ) )    m_ui.outputSuffixLineEdit->setText ( settings.value ( "outputSuffix" ).toString ( ) );
    if ( settings.contains ( "outputFormatChange" ) && settings.contains ( "outputFormatValue" ) )
    {
        if ( settings.value ( "outputFormatChange" ).toBool ( ) )
            m_ui.formatComboBox->setImageFormat ( settings.value ( "outputFormatValue" ).toString ( ) );
        else
            m_ui.formatComboBox->setImageFormat ( );
    }

    settings.endGroup ( );
}

