// Copyright Vrai Stacey, 2008
// $Id: filelistmodel.cpp 1428 2008-10-07 13:07:05Z vrai $

#include "filelistmodel.h"
#include <QSet>
#include <QMutexLocker>
#include <QtDebug>
#include <algorithm>

FileListModel::FileListModel ( QObject * parent )
    : QStringListModel ( parent )
{
}

QFileInfo FileListModel::getFileInfo ( int row ) const
{
    QMutexLocker locker ( &m_lock );
    QStringList filenames ( stringList ( ) );
    return QFileInfo ( filenames [ row ] );
}

void FileListModel::addFile ( const QFileInfo & file )
{
    // Place the fileinfo in a single element list - that way the addFiles code can be reused
    QList<QFileInfo> files;
    files.append ( file );
    addFiles ( files );
}

void FileListModel::addFiles ( const QList<QFileInfo> & files )
{
    QMutexLocker locker ( &m_lock );
    QStringList filenames ( stringList ( ) );

    // If the filename isn't already in the model - add it to both the index and the end of the filelist
    foreach ( const QFileInfo & file, files )
    {
        const QString filename ( file.absoluteFilePath ( ) );
        if ( m_filenameIndex.contains ( filename ) )
            continue;

        filenames += filename;
        m_filenameIndex.insert ( filename );
    }

    // Set the newly expanded list as the model contents and signal that the count has changed
    setStringList ( filenames );
    emit fileCountChanged ( rowCount ( ) );
}

void FileListModel::removeAll ( )
{
    // Clear both the index and model's content, signalling that the count has changed
    QMutexLocker locker ( &m_lock );
    m_filenameIndex.clear ( );
    setStringList ( QStringList ( ) );
    emit fileCountChanged ( rowCount ( ) );
}

void FileListModel::removeRows ( const QVector<int> & rows )
{
    QMutexLocker locker ( &m_lock );
    QStringList filenames ( stringList ( ) );

    // The row indicies must be in the correct (i.e. ascending) order. To ensure this they are copied in to a STL
    // vector which can then be sorted.
    std::vector<int> stlRows ( rows.toStdVector ( ) );
    std::sort ( stlRows.begin ( ), stlRows.end ( ) );

    // Must remove the rows in reverse order, otherwise the indicies will become invalid as the deletion occurs.
    // Remove the filenames from both the list and the index.
    for ( std::vector<int>::const_reverse_iterator rowIt ( stlRows.rbegin ( ) ), rowEnd ( stlRows.rend ( ) ); rowIt != rowEnd; ++rowIt )
    {
        m_filenameIndex.remove ( filenames [ *rowIt ] );
        filenames.removeAt ( *rowIt );
    }

    setStringList ( filenames );
    emit fileCountChanged ( rowCount ( ) );
}

