// Copyright Vrai Stacey, 2008
// $Id: filesearcher.cpp 1456 2008-11-08 12:30:49Z vrai $

#include "filesearcher.h"
#include "imageformats.h"
#include <QCoreApplication>
#include <QtDebug>

FileSearcher::FileSearcher ( QObject * parent )
    : QObject ( parent )
    , m_abort ( false )
    , m_running ( false )
    , m_filesChecked ( 0 )
{
}

FileSearcher::~FileSearcher ( )
{
}

void FileSearcher::run ( )
{
    if ( m_running )
    {
        qCritical ( ) << "Called to FileSearcher::run while searcher is already running";
        return;
    }

    // Clear the running state
    m_filesChecked = 0;
    m_visitedDirectories.clear ( );
    m_matchingFiles.clear ( );

    // Start the search
    m_running = true;
    m_abort = false;
    emit started ( );
    
    emit updateCount ( 0, 0 );
    searchDirectory ( m_rootDirectory.absolutePath ( ) );
    emit updateCount ( m_filesChecked, m_matchingFiles.size ( ) );

    // Search has finished - tell the world
    m_running = false;
    emit finished ( );
}

void FileSearcher::getMatchingFiles ( QList<QFileInfo> & matches ) const
{
    matches.clear ( );
    matches = m_matchingFiles;
}

void FileSearcher::configure ( const QDir & rootDirectory )
{
    if ( m_running )
    {
        qCritical ( ) << "Called to FileSearcher::configure while searcher is running";
        return;
    }

    m_rootDirectory = rootDirectory;
    ImageFormats::instance ( ).getAllExtensions ( m_validExtensions );
}

void FileSearcher::abort ( )
{
    m_abort = true;
}

void FileSearcher::searchDirectory ( const QString & directory )
{
    QCoreApplication::processEvents ( );

    if ( m_abort )
        return;

    // Only vist each directory once
    if ( m_visitedDirectories.contains ( directory ) )
        return;
    m_visitedDirectories.insert ( directory );

    // Tell any observer about the new directory - process any events this may cause
    emit updateDirectory ( directory );
    QCoreApplication::processEvents ( );

    // To keep the application responsive, process any pending events every few files
    const unsigned long maxFilesBetweenEvents ( 16 );
    unsigned long filesSinceLastEvents ( 0 );

    // Search this directory - add matching image files to the list and keep a list of all the subdirectories
    QFileInfoList subdirectories;
    foreach ( const QFileInfo & fileInfo, QDir ( directory ).entryInfoList ( ) )
    {
        if ( m_abort )
            return;

        // Ignore hidden files - this also prevents ".." causing infinite recursion issues under Unix
        if ( fileInfo.isHidden ( ) )
            continue;

        if ( fileInfo.isDir ( ) )
            subdirectories.append ( fileInfo );
        else
        {
            // Update the files visited count and process any pending events if sufficent files have been seen since
            // the last update.
            ++m_filesChecked;
            if ( ++filesSinceLastEvents > maxFilesBetweenEvents )
            {
                filesSinceLastEvents = 0;
                emit updateCount ( m_filesChecked, m_matchingFiles.size ( ) );
                QCoreApplication::processEvents ( );
            }

            // If the file matches our criteria, add it to the list
            if ( fileInfo.isFile ( ) && m_validExtensions.contains ( fileInfo.suffix ( ) ) )
                m_matchingFiles.append ( fileInfo );
        }
    }

    // Now recursive through the subdirectories
    foreach ( const QFileInfo & fileInfo, subdirectories )
        searchDirectory ( fileInfo.canonicalFilePath ( ) );
}

