// Copyright Vrai Stacey, 2008
// $Id: helpbrowser.cpp 1442 2008-11-01 09:51:40Z vrai $

#include "helpbrowser.h"
#include "imageformats.h"
#include <QResource>
#include <QRegExp>
#include <QtDebug>

// Visual Studio doesn't like the two percent characters used as token limiters
#ifdef Q_CC_MSVC
#   pragma warning ( disable: 4129 )
#endif
        
HelpBrowser::HelpBrowser ( QWidget * parent )
    : QTextBrowser ( parent )
    , m_pageIndex ( 0 )
{
}

void HelpBrowser::backward ( )
{
    // Cannot continue if already at the start of the page stack or the previous page cannot be loaded
    if ( m_pageIndex < 1 || ! loadPage ( m_pageStack [ m_pageIndex - 1 ] ) )
        return;
    
    // Previous page has been loaded, update the stack index and the navigation buttons
    --m_pageIndex;
    updateNavAvailability ( );
}

void HelpBrowser::forward ( )
{
    // Cannot continue if already at the end of the page stack or the next page cannot be loaded
    if ( m_pageIndex + 1 >= m_pageStack.size ( ) || ! loadPage ( m_pageStack [ m_pageIndex + 1 ] ) )
        return;

    // Next page has been loaded, update the stack index and the navigation buttons
    ++m_pageIndex;
    updateNavAvailability ( );
}

void HelpBrowser::setSource ( const QUrl & name )
{
    // Don't bother loading the page if we're already displaying it
    if ( ! m_pageStack.isEmpty ( ) && name == m_pageStack [ m_pageIndex ] )
        return;

    // Don't continue if the requested page cannot be loaded
    if ( ! loadPage ( name ) )
        return;

    // Update the page stack. If the stack is empty (this is the first call to setSource) then simply initialise the
    // stack index, otherwise cull any pages "forward" of the current one and increment the stack index.
    if ( m_pageStack.isEmpty ( ) )
        m_pageIndex = 0;
    else
    {
        while ( m_pageStack.size ( ) > m_pageIndex + 1 )
            m_pageStack.removeLast ( );
        ++m_pageIndex;
    }

    // Add the current page to end of the stack and update the navigation button availability
    m_pageStack.append ( name );
    updateNavAvailability ( );
}

bool HelpBrowser::loadPage ( const QUrl & name )
{
    QUrl url ( name );

    // Make sure that the URL is valid and is of the correct scheme
    if ( ! url.isValid ( ) )
    {
        qWarning ( ) << "HelpBrowser cannot set source \"" + url.toString ( ) + "\" - URL is not valid: " + url.errorString ( );
        return false;
    }

    if ( url.scheme ( ) != "help" )
    {
        qWarning ( ) << "HelpBrowser cannot load resources of scheme \"" + url.scheme ( ) + "\"";
        return false;
    }

    // Remove the fragment from the URL - but keep a copy for when the page is loaded
    QString fragment;
    if ( url.hasFragment ( ) )
    {
        fragment = url.fragment ( );
        url.setFragment ( "" );
    }

    // Use the URL to construct a resource name and check that it's available
    const QString resourceName ( QString ( ":/docs/help/%1.html" ).arg ( url.path ( ) ) );
    const QResource resource ( resourceName );
    if ( ! resource.isValid ( ) )
    {
        qWarning ( ) << "HelpBrowser cannot load resource \"" + resourceName + "\" - resource not valid";
        return false;
    }

    // Load the resource in to local memory and decompress it if required
    QByteArray data ( reinterpret_cast<const char *> ( resource.data ( ) ), resource.size ( ) );
    if ( resource.isCompressed ( ) )
        data = qUncompress ( data );

    // The data array should now contain uncompressed HTML - update the display
    QString html ( data );
    setHtml ( processHtml ( html ) );

    // If there was a fragment - scroll to the anchor
    if ( ! fragment.isEmpty ( ) )
        scrollToAnchor ( fragment );

    return true;
}

void HelpBrowser::updateNavAvailability ( )
{
    emit backwardAvailable ( m_pageIndex > 0 );
    emit forwardAvailable ( m_pageStack.size ( ) > m_pageIndex + 1 );
}

QString & HelpBrowser::processHtml ( QString & html ) const
{
    if ( QRegExp ( "\%\%[^\%]+\%\%" ).indexIn ( html ) >= 0 )
    {
        html = html.replace ( "%%BUILDTIME%%", QString ( "Built %1 %2" ).arg ( __DATE__ ).arg (  __TIME__ ) );
        html = html.replace ( "%%QTVERSION%%", QString ( "Qt %1" ).arg ( qVersion ( ) ) );
        html = html.replace ( "%%IMAGEFORMATS%%", ImageFormats::instance ( ).getAllFormats ( ).join ( " " ) );
    }

    return html;
}

