// Copyright Vrai Stacey, 2008
// $Id: imageformats.cpp 1467 2008-11-18 21:57:44Z vrai $

#include "imageformats.h"
#include <QImageReader>
#include <QTextStream>
#include <QtDebug>

ImageFormats * ImageFormats::s_instance = 0;
const char * ImageFormats::s_sharedExtensions [ ] = { "jpeg=jpeg;jpg;jif;jpe;jp2" };

const ImageFormats & ImageFormats::instance ( )
{
    if ( ! s_instance )
        s_instance = new ImageFormats;

    return *s_instance;
}

ImageFormats::ImageFormats ( )
{
    // Construct two lookups for the "shared" extensions (file formats for which there is more than one common file
    // extension). The first is a mapping of extension to type, the second a mapping of type to common extensions.
    QHash<QString, QString> extensionToType;
    StringToSetHash typeToExtensions;
    for ( int seIndex ( 0 ), seCount ( sizeof ( s_sharedExtensions ) / sizeof ( const char * ) ); seIndex < seCount; ++seIndex )
    {
        const QStringList elements ( QString ( s_sharedExtensions [ seIndex ] ).split ( "=" ) );
        foreach ( const QString & value, elements [ 1 ].split ( ";" ) )
        {
            extensionToType [ value ] = elements [ 0 ];
            typeToExtensions [ elements [ 0 ] ].insert ( value );
        }
    }

    // Read in the known formats from the Qt image reader. These will be combined with the shared extensions to build
    // a mapping of type to file extension.
    foreach ( const QByteArray & formatBytes, QImageReader::supportedImageFormats ( ) )
    {
        const QString format ( formatBytes );
        const QString lcFormat ( format.toLower ( ) );

        if ( extensionToType.contains ( lcFormat ) )
        {
            // This format has shared extensions - add all of them to the mapping
            const QString & type ( extensionToType [ lcFormat ] );
            if ( ! m_formats.contains ( type ) )
                foreach ( const QString & ext, typeToExtensions [ type ] )
                    m_formats [ type ].insert ( ext );
            m_formats [ type ].insert ( format );
        }
        else
            m_formats [ lcFormat ].insert ( format );
    }
}

ImageFormats::~ImageFormats ( )
{
}

bool ImageFormats::isFormatJpeg ( QString format ) const
{
    format = format.toLower ( );

    if ( m_formats.contains ( "jpeg" ) )
        return m_formats [ "jpeg" ].contains ( format );
    return false;
}

bool ImageFormats::isFormatJpeg ( const QByteArray & formatBytes ) const
{
    return isFormatJpeg ( QString ( formatBytes ) );
}

QString ImageFormats::getFileDialogConfig ( ) const
{
    QString output;
    QTextStream stream ( &output );

    // Build up a list of all extensions as we go - will be used to create an "All Supported" option
    StringSet allExtensions;

    for ( StringToSetHash::const_iterator it ( m_formats.begin ( ) ), begin ( m_formats.begin ( ) ), end ( m_formats.end ( ) ); it != end; ++it )
    {
        const StringSet & values ( it.value ( ) );
        allExtensions.unite ( values );

        // Don't bother adding any formats that have no extensions (not that any such formats should have got into the list)
        if ( values.empty ( ) )
            continue;
        // Separate the file types using double semi-colons
        if ( it != begin )
            stream << ";;";

        createFileDialogElement ( stream, it.key ( ).toUpper ( ), values );
    }

    // Add the catch all filter to the end of the list
    if ( ! output.isEmpty ( ) )
        stream << ";;";
    stream << "All files (*)";

    // Add the all supported formats options to the start of the string
    if ( ! allExtensions.isEmpty ( ) )
    {
        QString prefix;
        QTextStream prefixStream ( &prefix );

        createFileDialogElement ( prefixStream, "All Supported", allExtensions );
        prefixStream << ";;";

        return prefix + output;
    }
    else
        return output;
}

void ImageFormats::createFileDialogElement ( QTextStream & stream, const QString & type, const StringSet & values ) const
{
    // Add the file type and its list of extensions
    stream << type << " files (";
    for ( StringSet::const_iterator extIt ( values.begin ( ) ), extBegin ( values.begin ( ) ), extEnd ( values.end ( ) ); extIt != extEnd; ++extIt )
    {
        // Separate extensions using a single space
        if ( extIt != extBegin )
            stream << " ";
        // Assume that we want all files for each extension
        stream << "*." << *extIt;
    }

    stream << ")";
}

void ImageFormats::getAllExtensions ( QSet<QString> & extensions ) const
{
    extensions.clear ( );

    for ( StringToSetHash::const_iterator it ( m_formats.begin ( ) ), begin ( m_formats.begin ( ) ), end ( m_formats.end ( ) ); it != end; ++it )
        extensions.unite ( it.value ( ) );
}

QString ImageFormats::getExtension ( const QString & format ) const
{
    StringToSetHash::const_iterator it ( m_formats.find ( format ) );
    if ( it == m_formats.end ( ) || it.value ( ).empty ( ) )
        return format;

    // Jpegs are a special case - they have a number of extensions, some of which we don't want to use because of
    // compatilbilty problems with viewer software
    if ( isFormatJpeg ( format ) )
        return "jpeg";
    return *( it.value ( ).begin ( ) );
}

QStringList ImageFormats::getAllFormats ( ) const
{
    return m_formats.keys ( );
}

