// Copyright Vrai Stacey, 2008
// $Id: imageresizer.cpp 1467 2008-11-18 21:57:44Z vrai $

#include <QMutexLocker>
#include <QImageReader>
#include <QImageWriter>
#include <QImage>
#include <QCoreApplication>
#include "imageresizer.h"
#include "imageformats.h"
#include <QtDebug>

// Include the static image format plugins (if using a static build of QT)
#if ! defined( QT_SHARED ) && ! defined ( QT_DLL )
    Q_IMPORT_PLUGIN( qjpeg )
    Q_IMPORT_PLUGIN( qgif )
    Q_IMPORT_PLUGIN( qtiff )
#endif

ImageResizer::ImageResizer ( QObject * parent )
    : QObject ( parent )
    , m_running ( false )
    , m_abort ( false )
    , m_jpegQuality ( false )
    , m_verboseLogging ( false )
{
}

ImageResizer::~ImageResizer ( )
{
}

void ImageResizer::run ( )
{
    if ( m_running )
    {
        qCritical ( ) << "Call to ImageResizer::run while resizer is already running";
        return;
    }

    // Clear the error messages
    m_errors.clear ( );

    // Start the resizing
    m_running = true;
    m_abort = false;
    emit started ( );

    // Process each file in turn
    for ( int index ( 0 ), total ( m_files.size ( ) ); index < total && ! m_abort; ++index )
    {
        emit updateCount ( index + 1, total );
        QCoreApplication::processEvents ( );

        resizeImage ( m_files [ index ] );
    }

    // Resizing has finished
    m_running = false;
    emit finished ( );
}

void ImageResizer::getErrors ( QHash<QString, QString> & errors ) const
{
    errors = m_errors;
}

void ImageResizer::configure ( const QSize & targetDimensions,
                               const QDir & outputDirectory,
                               const QString & prefix,
                               const QString & suffix,
                               Qt::TransformationMode scalingMode,
                               RatioStrategy::Strategy ratioStrategy,
                               int jpegQuality )
{
    if ( m_running )
    {
        qCritical ( ) << "Called to ImageResizer::configure while thread is running";
        return;
    }
        
    m_targetDimensions = targetDimensions;
    m_outputDirectory = outputDirectory;
    m_prefix = prefix;
    m_suffix = suffix;
    m_scalingMode = scalingMode;
    m_ratioStrategy = ratioStrategy;
    m_jpegQuality = jpegQuality;
    m_format = qMakePair ( false, QString ( ) );
}

void ImageResizer::setFormatOverride ( const QString & format )
{
    m_format = qMakePair ( true, format );
}

void ImageResizer::clearFiles ( )
{
    m_files.clear ( );
    m_errors.clear ( );
}

void ImageResizer::addFile ( const QFileInfo & file )
{
    m_files.append ( file );
}

void ImageResizer::abort ( )
{
    m_abort = true;
}

QString ImageResizer::generateNewFilename ( const QFileInfo & file, const QByteArray & sourceFormat, const QByteArray & targetFormat )
{
    // Use the current extension if the format isn't to change - otherwise get the correct extension for the new format
    QString extension ( file.suffix ( ) );
    if ( sourceFormat != targetFormat )
        extension = ImageFormats::instance ( ).getExtension ( targetFormat );

    return QDir::cleanPath ( m_outputDirectory.absoluteFilePath ( m_prefix + file.baseName ( ) + m_suffix + "." + extension ) );
}

QByteArray ImageResizer::getOutputFormat ( const QByteArray & inputFormat )
{
    return m_format.first ? m_format.second.toUtf8 ( ) : inputFormat;
}

void ImageResizer::resizeImage ( const QFileInfo & file )
{
    const QString oldFilename ( file.filePath ( ) );
    QImageReader imageReader ( oldFilename );

    // Check that the image reader can handle the file. If not, log the relevant error and give up.
    if ( ! imageReader.canRead ( ) )
    {
        if ( QFile::exists ( oldFilename ) )
        {
            QByteArray format ( QImageReader::imageFormat ( oldFilename ) );
            if ( format.isEmpty ( ) )
                addError ( file, tr ( "Image not of a recognised format" ) );
            else
                addError ( file, tr ( "Image is a recognised format (%1) but was unloadable" ).arg ( QString ( format ) ) );
        }
        else
            addError ( file, tr ( "File does not exist" ) );
        return;
    }
    
    // Get the format of the source image and what will be used for the resized image
    const QByteArray format ( imageReader.format ( ) );
    const QByteArray targetFormat ( getOutputFormat ( format ) );

    // Construct the new filename and check that it doesn't exist - do this now so that we don't go to the expense of
    // resizing an image we won't write.
    const QString newFilename ( generateNewFilename ( oldFilename, format, targetFormat ) );
    if ( QFile::exists ( newFilename ) )
    {
        addError ( file, tr ( "Will not overwrite \"%1\"" ).arg ( newFilename ) );
        return;
    }
            
    // Load the image in to memory, again logging and bailing on failure.
    QImage image ( imageReader.read ( ) );
    if ( image.isNull ( ) )
    {
        addError ( file, tr ( "Could not load file: %1" ).arg ( imageReader.errorString ( ) ) );
        return;
    }

    // Loading a large image could have taken a while - process any pending events to keep the GUI responsive
    QCoreApplication::processEvents ( );
    if ( m_abort )
        return;
            
    // Resize the image
    const QSize oldDimensions ( image.size ( ) );
    image = image.scaled ( RatioStrategy::get ( m_ratioStrategy ) ( oldDimensions, m_targetDimensions ), Qt::IgnoreAspectRatio, m_scalingMode );

    // Again - resizing can be slow
    QCoreApplication::processEvents ( );
    if ( m_abort )
        return;

    // Write the file - only JPEG files have anything other than 100% quality
    QImageWriter imageWriter ( newFilename, targetFormat );
    imageWriter.setQuality ( ImageFormats::instance ( ).isFormatJpeg ( targetFormat ) ? m_jpegQuality : 0 );
    if ( ! imageWriter.write ( image ) )
        addError ( file,  tr ( "Could not write \"%1\": \"%2\"" ).arg ( newFilename ).arg ( imageWriter.errorString ( ) ) );
}

void ImageResizer::addError ( const QFileInfo & file, const QString & error )
{
    m_errors [ file.filePath ( ) ] = error;
}

